@@ -3,12 +3,13 @@ package Hades;
 use 5.006;
 use strict;
 use warnings;
-our $VERSION = '0.14';
+our $VERSION = '0.15';
 use Module::Generate;
 use Switch::Again qw/switch/;
 
 sub new {
 	my ($class, %args) = (shift, scalar @_ == 1 ? %{$_[0]} : @_);
+	$args{macros} = {} if !$args{macros};
 	bless \%args, $class;
 }
 
@@ -87,8 +88,10 @@ sub run {
 									? $self->build_our($mg, $token, $meta)
 									: $self->build_sub_or_accessor($mg, $token, $meta);
 			}
-			$self->build_new($mg, $meta);
-			$self->can('after_class') && $self->after_class($mg, $meta);
+			if (scalar keys %{$meta}) {
+				$self->build_new($mg, $meta);
+				$self->can('after_class') && $self->after_class($mg, $meta);
+			}
 		}
 	}
 	$self->can('before_generate') && $self->before_generate($mg);
@@ -100,8 +103,8 @@ sub build_class {
 	my ($self, $mg, $class) = @_;
 	if ($class->[0] eq 'macro') {
 		shift @{$class};
-		$mg->macro(shift @{$_}, join(' ', @{$_}) . ';') for @{$class};
-		next;
+		$self->build_macro($mg, $class);
+		return [];
 	}
 	while ($class->[0] =~ m/^(dist|lib|tlib|realm|author|email|version)$/) {
 		$mg->$1($class->[1]);
@@ -159,10 +162,7 @@ sub build_sub_no_arguments {
 	my $name = shift @{$token};
 	$name =~ m/^(begin|unitcheck|check|init|end|new)$/
 		? $mg->$name(join ' ', @{$token})
-		: $mg->sub($name)->code(sprintf qq|{
-			my (\$self) = \@_;
-			%s
-		}|, join ' ', @{$token})
+		: $mg->sub($name)->code($self->build_code($mg, $name, $self->build_sub_code($name, '', '', join ' ', @{$token})))
 		->pod(qq|call $name method. Expects no params.|)->example(qq|\$obj->$name()|);
 	return $meta;
 }
@@ -322,7 +322,7 @@ sub build_accessor {
 	my $type = $self->build_coerce($name, '$value', $meta->{$name}->{coerce})
 	. $self->build_type($name, $meta->{$name}->{type}[0]);
 	my $trigger = $self->build_trigger($name, '$value', $meta->{$name}->{trigger});
-	my $code = $self->build_accessor_code($name, $private, $type, $trigger);
+	my $code = $self->build_code($mg, $name, $self->build_accessor_code($name, $private, $type, $trigger));
 	$mg->accessor($name)->code($code)->clear_tests->test($self->build_tests($name, $meta->{$name}));
 	$meta->{$name}->{$_} && $mg->$_($self->replace_pe_string($meta->{$name}->{$_}, $name)) for qw/pod example/;
 }
@@ -349,13 +349,18 @@ sub build_modify {
 	my $before_code = $meta->{$name}->{before} || "";
 	my $around_code = $meta->{$name}->{around} || qq|my \@res = \$self->\$orig(\@params);|;
 	my $after_code = $meta->{$name}->{after} || "";
-	my $code = qq|{
+	my $code = $self->build_code($mg, $name, $self->build_modify_code($name, $before_code, $around_code, $after_code));
+	$mg->sub($name)->code($code)->pod(qq|call $name method.|)->test($self->build_tests($name, $meta->{$name}));
+	$meta->{$name}->{$_} && $mg->$_($self->replace_pe_string($meta->{$name}->{$_}, $name)) for qw/pod example/;
+}
+
+sub build_modify_code {
+	my ($self, $name, $before_code, $around_code, $after_code) =@_;
+	return qq|{
 		my (\$orig, \$self, \@params) = ('SUPER::$name', \@_);
 		$before_code$around_code$after_code
 		return wantarray ? \@res : \$res[0];
 	}|;
-	$mg->sub($name)->code($code)->pod(qq|call $name method.|)->test($self->build_tests($name, $meta->{$name}));
-	$meta->{$name}->{$_} && $mg->$_($self->replace_pe_string($meta->{$name}->{$_}, $name)) for qw/pod example/;
 }
 
 sub build_sub {
@@ -391,7 +396,7 @@ sub build_sub {
 		}
 	}
 	$meta->{$name}->{params_explanation} = $params_explanation;
-	$code = $self->build_sub_code($name, $params, $subtype, $code);
+	$code = $self->build_code($mg, $name, $self->build_sub_code($name, $params, $subtype, $code));
 	$params =~ s/^,\s*//;
 	my $example = qq|\$obj->$name($params)|;
 	$mg->sub($name)->code($code)
@@ -401,6 +406,29 @@ sub build_sub {
 	$meta->{$name}->{$_} && $mg->$_($self->replace_pe_string($meta->{$name}->{$_}, $name)) for qw/pod example/;
 }
 
+sub build_code {
+	my ($self, $mg, $name, $code) = @_;
+	return unless defined $code;
+	$code =~ s/€(.*);/$self->build_macro_code($mg, $1)/eg;
+	$code =~ s/£(\w*(\s|\$|\-|\;|\,|\{|\}|\[|\]|\)|\(|\:))/$self->build_self($1)/eg;
+	return $code;
+}
+
+sub build_self {
+	my ($self, $name) = @_;
+	return qq|\$self->$name|;
+}
+
+sub build_macro_code {
+	my ($self, $mg, $match) = @_;
+	if ($match =~ m/^(.*)\((.*)\)$/) {
+		return '' unless $self->{macros}->{$1}->{code};
+		return $self->{macros}->{$1}->{code}->($self, $mg, eval qq|($2)|);
+	} 
+	return '' unless $self->{macros}->{$match}->{code};
+	return $self->{macros}->{$match}->{code}->($self, $mg);	
+}
+
 sub build_sub_code {
 	my ($self, $name, $params, $subtype, $code) = @_;
 	return qq|{
@@ -412,11 +440,7 @@ sub build_sub_code {
 sub build_clearer {
 	my ($self, $mg, $name, $meta) = @_;
 	$mg->sub(qq|clear_$name|)
-		->code(qq|{
-			my (\$self) = \@_;
-			delete \$self->{$name};
-			return \$self;
-		}|)
+		->code($self->build_code($mg, $name, $self->build_clearer_code($name)))
 		->pod(qq|clear $name accessor|)
 		->example(qq|\$obj->clear_$name|)
 		->test(
@@ -426,13 +450,19 @@ sub build_clearer {
 		);
 }
 
+sub build_clearer_code {
+	my ($self, $name) = @_;
+	return qq|{
+		my (\$self) = \@_;
+		delete \$self->{$name};
+		return \$self;
+	}|;
+}
+
 sub build_predicate {
 	my ($self, $mg, $name, $meta) = @_;
 	$mg->sub(qq|has_$name|)
-		->code(qq|{
-			my (\$self) = \@_;
-			return exists \$self->{$name};
-		}|)
+		->code($self->build_code($mg, $name, $self->build_predicate_code($name)))	
 		->pod(qq|has_$name will return true if $name accessor has a value.|)
 		->example(qq|\$obj->has_$name|)
 		->test(
@@ -445,6 +475,14 @@ sub build_predicate {
 		);
 }
 
+sub build_predicate_code {
+	my ($self, $name) = @_;
+	return qq|{
+		my (\$self) = \@_;
+		return exists \$self->{$name};
+	}|;
+}
+
 sub build_coerce {
 	my ($self, $name, $param, $code) = @_;
 	return defined $code ? $code =~ m/^\w+$/
@@ -715,6 +753,57 @@ sub extend_error_string {
 	return $new_error_string;
 }
 
+sub build_macro_attributes {
+	my ($self, $name, $token, $meta) = @_;
+	return (
+		'default' => sub {
+			my $value = shift;
+			push @{$meta->{$name}->{caught}}, $value;
+		},
+		qr/^(\:a|\:alias)/ => sub {
+			my $value = shift;
+			$value =~ s/^\:(a|alias)\(\s*(.*)\s*\)$/$2/sg;
+			push @{$meta->{$name}->{alias}}, split(' ', $value);
+		},
+		qr/^(\{)/ => sub {
+			my $value = shift;
+			$value =~ s/^\{|\}$//g;
+			$meta->{$name}->{code} = eval qq|sub { my (\$self, \$mg, \@params) = \@_; $value }|;
+		},
+	);
+}
+
+sub build_macro {
+	my ($self, $mg, $class) = @_;
+
+	my $meta = $self->{macros};
+	for my $macro (@{$class}) {
+		if ($macro->[-1] !~  m/^{/) {
+			my $include = sprintf "Hades::Macro::%s", shift @{$macro};
+			eval qq|require $include|;
+			die $@ if $@;
+			my $include_meta = $include->new($macro->[0] ? do {
+				$macro->[0] =~ s/^\[|\]$//g;
+				( eval qq|$macro->[0]| ); 
+			} : ())->meta;
+			$meta = {%{$meta}, %{$include_meta}};
+		} else {
+			my $name = shift @{$macro};
+			$meta->{$name}->{meta} = 'MACRO';
+			my $switch = switch(
+				$self->build_macro_attributes($name, $macro, $meta)
+			);
+			$switch->(shift @{$macro}) while scalar @{$macro};
+			if ($meta->{$name}->{alias}) {
+				for (@{$meta->{$name}->{alias}}) {
+					$meta->{$_} = $meta->{$name};
+				}
+			}
+		}
+	}
+	$self->{macros} = $meta;
+}
+
 sub index {
 	my ($self, $index) = @_;
 	return substr $self->{eval}, $index, 1;
@@ -1083,7 +1172,7 @@ Hades - Less is more, more is less!
 
 =head1 VERSION
 
-Version 0.14
+Version 0.15
 
 =cut
 
@@ -1092,7 +1181,7 @@ Version 0.14
 	use Hades;
 
 	Hades->run({
-		eval => 'Kosmos { [penthos curae] :t(Int) :d(2) :p :pr :c :r geras $nosoi :t(Int) :d(2) { if ($self->penthos == $nosoi) { return $self->curae; } } }'
+		eval => 'Kosmos { [penthos curae] :t(Int) :d(2) :p :pr :c :r geras $nosoi :t(Int) :d(2) { if (£penthos == $nosoi) { return £curae; } } }'
 	});
 
 	... generates ...
@@ -1799,6 +1888,92 @@ Used in conjunction with Dict and Tuple to specify slots that are optional and m
 
 =cut
 
+=head2 Macros
+
+Hades has a concept of macros that allow you to write re-usable code. see L<https://metacpan.org/source/LNATION/Hades-0.15/macro-fh.hades> for an example of how to extend via macros.
+
+	macro {
+		FH [ macro => [qw/read_file write_file/], alias => { read_file => [qw/rf/], write_file => [qw/wf/] } ]
+		str2ArrayRef :a(s2ar) {
+			return qq|$params[0] = [ $params[0] ];|;
+		}
+		ArrayRef2Str :a(ar2s) {
+			return qq|$params[0] = $params[0]\->[0];|;
+		}
+	}
+	MacroKosmos {
+		eros $eros :t(Str) :d(t/test.txt) {
+			€s2ar('$eros');
+			€ar2s('$eros');
+			€wf('$eros', q|'this is a test'|);
+			return $eros;
+		}
+		psyche $psyche :t(Str) :d(t/test.txt) {
+			€rf('$psyche');
+			return $content;
+		}
+	}
+
+	... generates ...
+
+	package MacroKosmos;
+	use strict;
+	use warnings;
+	our $VERSION = 0.01;
+
+	sub new {
+		my ( $cls, %args ) = ( shift(), scalar @_ == 1 ? %{ $_[0] } : @_ );
+		my $self = bless {}, $cls;
+		my %accessors = ();
+		for my $accessor ( keys %accessors ) {
+			my $value
+			    = $self->$accessor(
+				defined $args{$accessor}
+				? $args{$accessor}
+				: $accessors{$accessor}->{default} );
+			unless ( !$accessors{$accessor}->{required} || defined $value ) {
+				die "$accessor accessor is required";
+			}
+		}
+		return $self;
+	}
+
+	sub eros {
+		my ( $self, $eros ) = @_;
+		$eros = defined $eros ? $eros : "t/test.txt";
+		if ( !defined($eros) || ref $eros ) {
+			$eros = defined $eros ? $eros : 'undef';
+			die qq{Str: invalid value $eros for variable \$eros in method eros};
+		}
+
+		$eros = [$eros];
+		$eros = $eros->[0];
+		open my $wh, ">", $eros or die "cannot open file for writing: $!";
+		print $wh 'this is a test';
+		close $wh;
+		return $eros;
+
+	}
+
+	sub psyche {
+		my ( $self, $psyche ) = @_;
+		$psyche = defined $psyche ? $psyche : "t/test.txt";
+		if ( !defined($psyche) || ref $psyche ) {
+			$psyche = defined $psyche ? $psyche : 'undef';
+			die
+			    qq{Str: invalid value $psyche for variable \$psyche in method psyche};
+		}
+
+		open my $fh, "<", $psyche or die "cannot open file for reading: $!";
+		my $content = do { local $/; <$fh> };
+		close $fh;
+		return $content;
+	}
+
+	1;
+
+	__END__
+
 =head2 Testing
 
 Hades can auto-generate test files. If you take the following example:
